@@ -390,7 +390,8 @@ ticklist *tick_for(coord *c, double from, double to, bydesc by, DisplayString *r
 		
     if ( to - from >= 0 ) dir = 1;
     else dir = -1;
-		
+
+    //  I think EPSILON is correct here.
     idx = from;
     while ( (idx - to) *dir  < EPSILON ) {
 	t = new tick(idx, 0, top_side, 0, (shiftlist *) 0, 0);
@@ -547,9 +548,11 @@ void line_statement(int is_line, linedesc *ld1, point *p1,
 	else
 	    the_graph->new_linesegment(p2->x, p2->y, p2->c, l, 0, &des, true);
     } else {
-	the_graph->new_linesegment(p1->x, p1->y, p1->c, l);
-	if (is_line) the_graph->new_linesegment(p2->x, p2->y, p2->c, l);
-	else the_graph->new_linesegment(p2->x,p2->y,p2->c, l, 0, 0, true);
+	the_graph->new_linesegment(p1->x, p1->y, p1->c, l, 0, &des);
+	if (is_line) 
+	    the_graph->new_linesegment(p2->x, p2->y, p2->c, l, 0, &des);
+	else 
+	    the_graph->new_linesegment(p2->x,p2->y,p2->c, l, 0, &des, true);
     }
     delete ld1; delete ld2;
     delete p1; delete p2;
@@ -661,9 +664,9 @@ void process_frame(linedesc* d, frame *f, frame *s) {
     int i;	// Scratch
 
     if ( d ) {
-	// a default linedesc is possible, but unlikely (only a fill,
-	// for example).
-	if ( d->ld != def ) {
+	// If the default linedesc has a description, color or thickness,
+	// propagate it.
+	if ( d->ld != def || d->color || d->thick != 0.0 ) {
 	    for ( i = 0 ; i < 4; i++ ) {
 		the_graph->base->desc[i] = *d;
 	    }
@@ -679,7 +682,10 @@ void process_frame(linedesc* d, frame *f, frame *s) {
 		
     if ( s ) {
 	for ( i = 0 ; i < 4; i++ ) {
-	    if ( s->desc[i].ld != def )
+	    // If the linedesc has a description, color or thickness, propagate
+	    // it.
+	    if ( s->desc[i].ld != def || s->desc[i].color ||
+		    s->desc[i].thick != 0.0  )
 		the_graph->base->desc[i] = (linedesc) s->desc[i];
 	}
 	delete s;
@@ -825,13 +831,20 @@ void usage() {
     cerr << "\t-M\tspecify search path for files" << endl;
     cerr << "\t-C\tcompatibility mode" << endl;
     cerr << "\t-S\tsafer mode, no internal user sprintf calls" << endl;
+    cerr << "Defines are in " << DEFINES << endl;
     cerr << "See the man page for more information." << endl;
+    cerr << "Documentation in " << DOCS_DIR << endl;
+    cerr << "Examples in " << EXAMPLES_DIR << endl;
     exit(5);
 }
 
 inline void version() {
-    cout << "grap " << PACKAGE_VERSION << " compiled under ";
-    cout << OS_VERSION << " report bugs to " << PACKAGE_BUGREPORT << endl;
+    cout << "grap " << PACKAGE_VERSION << " compiled under " 
+	 << OS_VERSION << endl;
+    cout << "Report bugs to " << PACKAGE_BUGREPORT << endl;
+    cout << "Documentation in " << DOCS_DIR << endl;
+    cout << "Examples in " << EXAMPLES_DIR << endl;
+    cout << "Defines are in " << DEFINES << endl;
     exit(5);
 }
 
